#ifndef SC_OUTSTREAM_HH_
#define SC_OUTSTREAM_HH_

#include <iostream>
#include <string>

#include "ell/SC_Counting.hh"

namespace ell
{

	/*! A WalkCollector that prints each reported state to the given outstream
	 * using the additional given flags.
	 * 
	 * @author Martin Mann <mmann@informatik.uni-freiburg.de>
	 */
	class SC_Outstream : public SC_Counting
	{
	protected:
		  //! the stream to write the states to
		std::ostream& out;
		
		  //! additional string for accepted states
		const std::string strAdded;
		
	public:
	
		  //! Constructs a WalkCollector that writes all States to out stream.
		  //! @param out the outstream to write to
		  //! @param labelAdded		label for accepted states
		SC_Outstream(	std::ostream& out, 
						const std::string& labelAdded);
		
		virtual ~SC_Outstream();
	
		  //! This function is used to track all accepted intermediate States.
		  //! @param s the accepted State
		virtual void add(const State& s);
			
	};

}

#endif /*SC_OUTSTREAM_HH_*/
