#ifndef WALKCOLLECTOR_HH_
#define WALKCOLLECTOR_HH_

#include "ell/State.hh"
#include <vector>

namespace ell 
{
	/*!
	 * This is an abstract interface used by all state collecting or processing
	 * classes (SC_*).
	 * 
	 * @author Daniel Maticzka
	 * @author Martin Mann
	 */
	class StateCollector
	{
	public:	
	
		virtual
		~StateCollector() {}
		
		  //! This function makes the collector manage the given State.
		  //! @param s the added State
		virtual void add(const State& s) = 0;
		
		  //! Returns number of added States.
		  //! @return the number of added states
		virtual size_t size() const = 0;
		
		  //! Returns last added State.
		  //! @return last added State
		virtual const State* const getLastAdded() const = 0;
		
	};
	
} // namespace ell

#endif /*WALKCOLLECTOR_HH_*/
