#include "ell/spinglass/S_SG_NPP.hh"
#include "biu/RandomNumberGenerator.hh"
#include "biu/RandomNumberFactory.hh"
#include <iostream>
#include <iterator>
#include <algorithm>
#include <set>


double getRN() {
	return ((double)biu::RNF::getRN()/(double)biu::RNF::getMaxRN());
}

int main(int argc, char **argv) {

	using namespace ell;

	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : S_SG_NPP\n"
				<<"===========================================" <<std::endl;
				
		// erzeugen	
	std::cout	<<"=> biu::RNF::setRNG( biu::RNG_LCG() )" <<std::endl;
	biu::RandomNumberGenerator* rng = new biu::RNG_LCG();
	biu::RNF::setRNG( rng );
	delete rng;
	std::cout	<<"=> biu::RNF::setSeed(100)" <<std::endl;
	biu::RNF::getRNG().setSeed(100);
	
	std::cout	<<"=> generate weights(10) :\n  "; std::cout.flush();
	S_SG_NPP::WeightType w(10);
	double (*pt_myRand)() = &getRN;
	std::generate (w.begin(),w.end(),pt_myRand);
	std::copy(w.begin(), w.end(), std::ostream_iterator<double>(std::cout, "\n  "));
	std::cout.flush();

	std::cout	<<"=> creating S_SG_NPP(10,false) = s1" <<std::endl;
		S_SG_NPP s1(&w,false);
	std::cout	<<"=> creating S_SG_NPP(10,true) = s2" <<std::endl;
		S_SG_NPP s2(&w,true);
	
		// toString()
	std::cout	<<"->s1.toString()  = "
				<<s1.toString()
				<<std::endl
				<<"->s2.toString()  = "
				<<s2.toString()
				<<std::endl;

		// getFitness()
	std::cout	<<"->s1.getFitness()  = "
				<<(s1.getFitness())
				<<std::endl;
	std::cout	<<"->s2.getFitness()  = "
				<<(s2.getFitness())
				<<std::endl;

		// getEnergy()
	std::cout	<<"->s1.getEnergy()  = "
				<<(s1.getEnergy())
				<<std::endl;
	std::cout	<<"->s2.getEnergy()  = "
				<<(s2.getEnergy())
				<<std::endl;

	ell::State* sc1 = s1.clone();

		// clone()  ==
	std::cout	<<"->(*this == *(this->clone()))  = "
				<<(s1 == *sc1 ? "true":"false")
				<<std::endl;

		// clone()  !=
	std::cout	<<"->(*this != *(this->clone()))  = "
				<<(s1 != *sc1 ? "true":"false")
				<<std::endl;
	delete sc1;
	
	s2.getRandomNeighbor(&s1);
	std::cout	<<"->s2.getRandomNeighbor(s1)"
				<<"\n  s2.toString()  = "
				<<s2.toString()
				<<"\n  s1.toString()  = "
				<<s1.toString()
				<<" E = " <<s1.getEnergy()
				<<std::endl;
	
	
		// getMinimalDistance(..)
	std::cout	<<"Distance s1 - s2 : "
				<<s1.getMinimalDistance(s2)
				<<std::endl;
	
	int num = 0;

		// getNeighborList()
	ell::State::NeighborListPtr nl = s1.getNeighborList();
	std::set< std::pair<std::string,double> > neighs, randneighs;
	std::set< std::pair<std::string,double> >::const_iterator setIt;
	std::cout	<<"->s1.getNeighborList()  ==> now sorted" <<std::endl;
	for(ell::State::NeighborList::Iterator it = nl->begin();
		 it != nl->end(); ++it) {
		neighs.insert(std::pair<std::string,double>(it->toString(),it->getEnergy()));
	}
		std::cout <<" s1" <<"\t: " <<s1.toString() <<std::endl;
	for(setIt = neighs.begin(); setIt != neighs.end(); setIt++) {
		std::cout <<" " <<++num <<"\t: " <<setIt->first <<"  E = " <<setIt->second <<std::endl;
	}


		// getRandomNeighborList()
	num = 0;
	nl = s1.getRandomNeighborList();
	std::cout	<<"->s1.getRandomNeighborList() ==> now sorted" <<std::endl;
	for(ell::State::NeighborList::Iterator it = nl->begin();
		 it != nl->end(); ++it) {
		randneighs.insert(std::pair<std::string,double>(it->toString(),it->getEnergy()));
	}
	for(setIt = randneighs.begin(); setIt != randneighs.end(); setIt++) {
		std::cout <<" " <<++num <<"\t: " <<setIt->first <<"  E = " <<setIt->second <<std::endl;
	}
	
	
	// TODO: test for new compress and uncompress functions	
//	ell::CState* c1 = s1.compressedClone();
//	ell::CState* c2 = s2.compressedClone();
//
//		// compressedClone()  ==   compress(..)
//	std::cout	<<"=>s1.compress(c2)" <<std::endl;
//	s1.compress(*c2);
//	std::cout	<<"->(*(s1.compressedClone()) == c2)  = "
//				<<(*c1 == *c2 ? "true":"false")
//				<<std::endl;
//
//		// compressedClone()  !=   compress(..)
//	std::cout	<<"->(*(s1.compressedClone()) != c2)  = "
//				<<(*c1 != *c2 ? "true":"false")
//				<<std::endl;
//
//	ell::CState* c3 = c2->clone();
//
//		// X  ==  *(X.clone())
//	std::cout	<<"->(c2 == *(c2.clone()))  = "
//				<<(*c2 == *c3 ? "true":"false")
//				<<std::endl;
//	delete c3;
//
//		// getCStateStringRepresentation()
//	std::cout	<<"->(c1->getCStateStringRepresentation())"
//				<<"  = "
//				<<c1->getCStateStringRepresentation()
//				<<std::endl;
//
//		// getUncomprStateStringRepresentation()
//	std::cout	<<"->(c1->getUncomprStateStringRepresentation())  = "
//				<<c1->getUncomprStateStringRepresentation()
//				<<std::endl;
//
//	ell::State* s3 = c1->uncompressedClone();
//
//		// uncompressedClone()
//	std::cout	<<"->(*(s1.compressedClone()->uncompressedClone()) == "
//				<<"*(s1))  = "
//				<<(*s3 == s1 ? "true":"false")
//				<<std::endl;
//
//	c1->uncompress(s2);
//
//		// uncompress(..)
//	std::cout	<<"=>s1.compressedClone()->uncompress(s2)" <<std::endl;
//	std::cout	<<"->(s1 == s2)  = "
//				<<(s1 == s2 ? "true":"false")
//				<<std::endl;
//
//	std::ofstream out("out.bin");
//	if (!out) {
//		std::cerr<<"out.bin couldn't be opened!"<<std::endl;
//		exit(-1);
//	}
//	c1->writeBinaryOutput(out);
//	out.close();
//	std::ifstream in("out.bin");
//	if (!in) {
//		std::cerr<<"out.bin couldn't be opened!"<<std::endl;
//		exit(-1);
//	}
//	c3 = c1->readBinaryInputClone(in);
//	in.close();
//
//		// writeBinaryOutput(..), readBinaryInputClone(..)
//	std::cout	<<"=>std::ofstream out(\"out.bin\")" <<std::endl;
//	std::cout	<<"=>c1.writeBinaryOutput(out)" <<std::endl;
//	std::cout	<<"=>std::ifstream in(\"out.bin\")" <<std::endl;
//	std::cout	<<"=>c3 = c1.readBinaryInputClone(in)" <<std::endl;
//	std::cout	<<"->(*c3 == *c1)  = "
//				<<(*c3 == *c1 ? "true":"false")
//				<<std::endl;
//
//	delete c1;
//	delete c2;
//	delete c3;
//	delete s3;
	
	// test fromString
	State* state = new S_SG_NPP(&w,false);
	std::cout 	<< "State* s_fromString = state->fromString(\"+-+-+-+-\")" 
				<< std::endl;
	State* s_fromString = state->fromString("+-+-+-+-");
	std::cout	<< "s_fromString->toString() = " << s_fromString->toString() 
				<< std::endl;	

	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	return 0;
}
