#include "ell/WalkAbortionCriterion.hh"
#include "ell/SC_Counting.hh"
#include "ell/State.hh"
#include <iostream>

using namespace ell;

/*!
 * This is a dummy implementation of a State class. The only functionality
 * implemented is setting an id and energy in the constructor and retrieving it via
 * toString and getEnergy.
 * 
 * @author Daniel Maticzka
 */
class DummyState : public State
{
	public:
	
	std::string id;
	double energy;

	DummyState(std::string _id, double _energy)
	{
		id = _id; 
		energy = _energy;
	}
	
	~DummyState()
	{}
	
	bool operator== (const State& state2) const {return true;}
	bool operator!= (const State& state2) const {return true;}

	int	getFitness() const 
	{
		return 0;
	}
	
	double getEnergy() const 
	{
		return energy;
	}
	
	unsigned int getMinimalDistance(const State& state2) const 
	{
		return 0;
	}
	
	std::string	toString() const 
	{
		return id;
	}

	NeighborListPtr getNeighborList() const
	{
		return *(new NeighborListPtr);
	}

	NeighborListPtr getRandomNeighborList() const
	{
		return *(new NeighborListPtr);
	}

	State* getRandomNeighbor(State* inPlaceNeigh = NULL) const
	{
		return NULL;
	}
	//////////////  COMPRESSED REPRESENTATION  ////////////////////////////
	
	CSequence  compress(void) const {
		return CSequence();
	}
	
	CSequence&   compress(CSequence& toFill) const {
		toFill.clear();
		return toFill;
	}
	
	State*   uncompress(const CSequence& cseq, State* toFill) const {
		return toFill;
	}
	
	State*   uncompress(const CSequence& cseq) {
		return NULL;
	}

	State* clone() const
	{
		return new DummyState(id,energy);
	}
	
	State* fromString(const std::string& stringRep) const { return NULL; }
	
};


int main(int argc, char** argv)
{
	
	/*
	 * Test WAC_MaxEnergy
	 */
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : WAC_MaxEnergy\n"
				<<"===========================================" <<std::endl;

	std::cout << "=> creating DummyState(\"State 1\", 5)" << std::endl;
	State* s = new DummyState("State 1", 5);
	std::cout << "->getEnergy = " << s->getEnergy() << std::endl;
	std::cout << "=> creating StateCollector sc" << std::endl;
	StateCollector* sc = new SC_Counting();
	std::cout << "->add(*s)" << std::endl;
	sc->add(*s);
	
	std::cout << "=> creating WAC_MaxEnergy(4)" << std::endl;
	WalkAbortionCriterion* wacMaxEnergy = new WAC_MaxEnergy(4);
	std::cout << "->abort(sc) = " << wacMaxEnergy->abort(sc) << std::endl;
	
	std::cout << "=> creating WAC_MaxEnergy(6)" << std::endl;
	delete wacMaxEnergy;
	wacMaxEnergy = new WAC_MaxEnergy(6);
	std::cout << "->abort(sc) = " << wacMaxEnergy->abort(sc) << std::endl;
	
	delete s;
	delete sc;
	delete wacMaxEnergy;
	
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	
	/*
	 * Test WAC_MinEnergy
	 */
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : WAC_MinEnergy\n"
				<<"===========================================" <<std::endl;
	std::cout << "=> creating DummyState(\"State 1\", 5)" << std::endl;
	s = new DummyState("State 1", 5);
	std::cout << "->getEnergy = " << s->getEnergy() << std::endl;
	std::cout << "=> creating StateCollector sc" << std::endl;
	sc = new SC_Counting();
	std::cout << "->add(*s)" << std::endl;
	sc->add(*s);
	
	std::cout << "=> creating WAC_MinEnergy(4)" << std::endl;
	WalkAbortionCriterion* wacMinEnergy = new WAC_MinEnergy(4);
	std::cout << "->abort(sc) = " << wacMinEnergy->abort(sc) << std::endl;
		
	std::cout << "=> creating WAC_MinEnergy(6)" << std::endl;
	delete wacMinEnergy;
	wacMinEnergy = new WAC_MinEnergy(6);
	std::cout << "->abort(sc) = " << wacMinEnergy->abort(sc) << std::endl;
	
	delete s;
	delete sc;
	delete wacMinEnergy;

	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	/*
	 * Test WAC_MaxLength
	 */
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : WAC_MaxLength\n"
				<<"===========================================" <<std::endl;
	std::cout << "=> creating DummyState(\"State 1\", 5)" << std::endl;
	s = new DummyState("State 1", 5);
	std::cout << "=> creating StateCollector sc" << std::endl;
	sc = new SC_Counting();
	std::cout << "->add(*s)" << std::endl;
	sc->add(*s);
	std::cout << "->add(*s)" << std::endl;
	sc->add(*s);
	std::cout << "->add(*s)" << std::endl;
	sc->add(*s);
	std::cout << "->add(*s)" << std::endl;
	sc->add(*s);
	std::cout << "->add(*s)" << std::endl;
	sc->add(*s);
	std::cout << "->size: " << sc->size() << std::endl;
	
	std::cout << "=> creating WAC_MaxLength(4)" << std::endl;
	WalkAbortionCriterion* wacMaxLength = new WAC_MaxLength(4);
	std::cout << "->abort(sc) = " << wacMaxLength->abort(sc) << std::endl;
		
	std::cout << "=> creating WAC_MaxLength(6)" << std::endl;
	delete wacMaxLength;
	wacMaxLength = new WAC_MaxLength(6);
	std::cout << "->abort(sc) = " << wacMaxLength->abort(sc) << std::endl;
	
	delete s;
	delete sc;
	delete wacMaxLength;

	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	return 0;
}
